package service

import (
	"context"
	"testing"

	"git.perx.ru/perxis/perxis-go/pkg/locales"
	"git.perx.ru/perxis/perxis-go/pkg/locales/mocks"
	"github.com/stretchr/testify/assert"
)

func Test_deliveryService_ListLocales(t *testing.T) {
	tests := []struct {
		name        string
		mockcall    func(locSvs *mocks.Locales)
		wantLocales []*locales.Locale
		wantErr     bool
	}{
		{
			name: "Returned locales without NoPublish and Disabled",
			mockcall: func(locSvs *mocks.Locales) {
				locSvs.On("List", context.Background(), "space").Return([]*locales.Locale{
					{ID: "en_EN", SpaceID: "space", Name: "english", Code: "en", NoPublish: false, Disabled: false},
					{ID: "de_DE", SpaceID: "space", Name: "deutsch", Code: "de", NoPublish: false, Disabled: false},
					{ID: "ru_RU", SpaceID: "space", Name: "russian", Code: "ru", NoPublish: false, Disabled: false},
					{ID: "by_BY", SpaceID: "space", Name: "belarus", Code: "by", NoPublish: true, Disabled: false},
					{ID: "es_ES", SpaceID: "space", Name: "spain", Code: "by", NoPublish: false, Disabled: true}}, nil)
			},
			wantLocales: []*locales.Locale{
				{ID: "en_EN", SpaceID: "space", Name: "english", Code: "en", NoPublish: false, Disabled: false},
				{ID: "de_DE", SpaceID: "space", Name: "deutsch", Code: "de", NoPublish: false, Disabled: false},
				{ID: "ru_RU", SpaceID: "space", Name: "russian", Code: "ru", NoPublish: false, Disabled: false},
			},
			wantErr: false,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			localesService := &mocks.Locales{}
			if tt.mockcall != nil {
				tt.mockcall(localesService)
			}
			s := deliveryService{
				Locales: localesService,
			}

			gotLocales, err := s.ListLocales(context.Background(), "space")
			if tt.wantErr {
				assert.Error(t, err)
				return
			}
			assert.NoError(t, err)
			assert.Equal(t, tt.wantLocales, gotLocales)
			localesService.AssertExpectations(t)
		})
	}
}
