package field

import (
	"context"
	"fmt"
	"reflect"
	"strconv"
	"testing"
)

type customString string

func TestStringField_Decode(t *testing.T) {
	tests := []struct {
		name    string
		field   *Field
		data    interface{}
		want    interface{}
		wantErr bool
	}{
		{"Correct", String(), "string", "string", false},
		{"Correct", String(), customString("string"), "string", false},
		{"Wrong data", String(), 2, nil, true},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := Decode(context.Background(), tt.field, tt.data)
			if (err != nil) != tt.wantErr {
				t.Errorf("Decode() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("Decode() got = %v, want %v", got, tt.want)
			}
		})
	}
}

func TestStringField_Encode(t *testing.T) {
	tests := []struct {
		name    string
		field   *Field
		data    interface{}
		want    interface{}
		wantErr bool
	}{
		{"Correct", String(), "string", "string", false},
		{"Wrong data", String(), 2, nil, true},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := Encode(context.Background(), tt.field, tt.data)
			if (err != nil) != tt.wantErr {
				t.Errorf("Decode() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("Decode() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// Добавлен полные функции, чтобы результаты теста не поменялись, в случае изменения Decode
// Старая реализация.
func decodeStringReflect(v interface{}) (interface{}, error) { //nolint:unparam // test case
	if v == nil {
		return nil, nil //nolint:nilnil // test case
	}
	t := reflect.ValueOf(v)
	if t.Kind() == reflect.String {
		return t.String(), nil
	}
	return nil, fmt.Errorf("StringField decode error: unsupported value type : \"%s\"", reflect.ValueOf(v).Kind())
}

// Новая реализация.
func decodeString(v interface{}) (interface{}, error) {
	if v == nil {
		return nil, nil //nolint:nilnil // test case
	}
	if _, ok := v.(string); ok {
		return v, nil
	}
	return nil, fmt.Errorf("StringField decode error: unsupported value type : \"%s\"", reflect.ValueOf(v).Kind())
}

// команда запуска go test -bench=.
func BenchmarkDecodeStringReflect(b *testing.B) {
	for i := 0; i < b.N; i++ {
		_, _ = decodeStringReflect(strconv.Itoa(i))
	}
}
func BenchmarkDecodeCustomStringReflect(b *testing.B) {
	for i := 0; i < b.N; i++ {
		_, _ = decodeStringReflect(customString(strconv.Itoa(i)))
	}
}
func BenchmarkDecodeString(b *testing.B) {
	for i := 0; i < b.N; i++ {
		_, _ = decodeString(strconv.Itoa(i))
	}
}
func BenchmarkDecodeStringCurrentSolution(b *testing.B) {
	for i := 0; i < b.N; i++ {
		_, _ = StringType{}.Decode(context.Background(), nil, strconv.Itoa(i))
	}
}
func BenchmarkDecodeCustomStringCurrentSolution(b *testing.B) {
	for i := 0; i < b.N; i++ {
		_, _ = StringType{}.Decode(context.Background(), nil, customString(strconv.Itoa(i)))
	}
}
